#!/usr/bin/env python3

import numpy as np
import Chunk
import random
from PerlinGenerator import PerlinGenerator

from typing import Tuple

class WorldManager(object):

    def __init__(self, world_seed:int, raw_noise_cell_size:int, raw_noise_chunk_size:int):

        self.world_seed = world_seed

        # create Perlin Generator
        self.pg = PerlinGenerator(raw_noise_cell_size=raw_noise_cell_size, raw_noise_chunk_size=raw_noise_chunk_size, world_seed=world_seed)

        # dictionary of dictionaries of dictionaries --> 3D hasmap (assume that's more efficient than using tuples as keys,
        # but future me should do further investigation into that [TODO!])
        self.chunks = {} 

        # 2Dimensional map. Will be 3D in future [TODO]
        self.perlin_maps = {}

        # create initial chunk
        self.createChunk(0,0,0)
    
    def isChunkKnown(self, chunk_x:int, chunk_y:int, chunk_z:int) -> bool:
        # TODO: this should be done more efficient!
        return chunk_x in self.chunks and chunk_y in self.chunks[chunk_x] and chunk_z in self.chunks[chunk_x][chunk_y]
    
    def createChunk(self, chunk_x:int, chunk_y:int, chunk_z:int) -> None:

        if not chunk_x in self.chunks:
            self.chunks[chunk_x] = {}
        
        if not chunk_y in self.chunks[chunk_x]:
            self.chunks[chunk_x][chunk_y] = {}
        
        self.chunks[chunk_x][chunk_y][chunk_z] = Chunk.Chunk(chunk_x,chunk_y,chunk_z)
    
    def hasPerlinMap(self, chunk_x:int, chunk_z:int) -> bool:
        return chunk_x in self.perlin_maps and chunk_z in self.perlin_maps[chunk_x]
    
    def __createPerlinMap(self, chunk_x:int, chunk_z:int) -> None:
        pm = self.perlin_maps[chunk_x][chunk_z] = np.zeros(shape=(Chunk.Chunk.CHUNK_SIDELENGTH, Chunk.Chunk.CHUNK_SIDELENGTH), dtype=float)

        x0:int = chunk_x * Chunk.Chunk.CHUNK_SIDELENGTH
        z0:int = chunk_z * Chunk.Chunk.CHUNK_SIDELENGTH

        for i,j in np.ndindex(pm.shape):
            pm[i,j] = self.pg.perlin(i + x0,j + z0)
    
    def getPerlinMap(self, x:int, z:int):
        if not x in self.perlin_maps:
            self.perlin_maps[x] = {}
        
        if z in self.perlin_maps[x]:
            return self.perlin_maps[x][z]
        
        self.__createPerlinMap(x,z)

        return self.perlin_maps[x][z]
        
        
        
        



